from importlib import import_module
import os
import subprocess

from fabric.api import sudo, local, put, settings

from .services.nginx import NGINX_USER
from .services.s3 import upload_to_s3, delete_from_s3
from .shared import *

class own_project(object):
	"""Changes the permissions of the project between the ssh user and nginx user."""
	def __init__(self):
		pass
	def __enter__(self):
		sudo('chown -R %s:%s /srv/www/%s' % (env['user'], env['user'], env['project']))
	def __exit__(self, *_):
		sudo('chown -R %s:%s /srv/www/%s' % (NGINX_USER, NGINX_USER, env['project']))

def _django_get_excluded(sites):
	"""Get an excluded list of patterns.

	rsync patterns are similar to .gitignore patterns. e.g. /foo matches foo at the base.
	http://git-scm.com/docs/gitignore
	http://ss64.com/bash/rsync.html
	"""
	excluded = ['.*', '*.pyc', '*.db']
	# Exclude anything additionally specified in the environment
	if env.has_key('excluded_files') and env['excluded_files']:
		excluded.extend(env['excluded_files'])
	# Exclude anything in the root .gitignore file
	if os.path.exists('.gitignore'):
		with file('.gitignore', 'r') as f:
			for line in f:
				line = line.strip()
				if line.startswith('.') or line.find('!') != -1:
					continue
				if line not in excluded:
					excluded.append(line)
	# Exlude all of the webapp's node_modules and bower_components
	for site in sites:
		module = import_module(site['settings_module'])
		os.environ.setdefault("DJANGO_SETTINGS_MODULE", site['settings_module'])
		if hasattr(module, 'WEBAPP_ROOT'):
			webapp_root = module.WEBAPP_ROOT
			try:
				parent, task_runner = get_webapp_taskrunner(webapp_root)
				if task_runner:
					excluded.append('/' + os.path.relpath(parent) + '/node_modules')
					excluded.append('/' + os.path.relpath(parent) + '/bower_components')
				# If wanting to exclude the entire webapp
				#if task_runner:
				#	excluded.append('/' + os.path.relpath(parent))
				#else:
				#	excluded.append('/' + os.path.relpath(webapp_root))
			except:
				print red('Warning: Could not import webapp settings for %s when syncing.' % site['name'])
	return excluded

SYNC_COMMAND = 'rsync -avz %s --delete --delete-excluded%s ./ %s@%s:/srv/www/%s'

def django_sync(sites):
	"""Syncs the local code to the server. Used as part of the deploy process. Wrap in the task decorator to enable. sync = task(sync)"""
	sudo('mkdir -p /srv/www/%s' % env['project'])
	with own_project():
		# Preserve existing media if a subdirectory of the project
		for site in sites:
			module = import_module(site['settings_module'])
			MEDIA_ROOT = fix_static_path(module.MEDIA_ROOT, site)
			if MEDIA_ROOT.startswith('/srv/www/%s/' % env['project']) and exists(MEDIA_ROOT):
				sudo('mv %s /srv/www/%smedia' % (MEDIA_ROOT, env['project']))

		# Sync and set the owner
		excluded = _django_get_excluded(sites)
		excluded = ['--exclude="%s"' % ex for ex in excluded]
		local(SYNC_COMMAND % (' '.join(excluded), ' -e "ssh -i %s"' % os.path.expanduser(env['key_filename']) if env.get('key_filename') else '', env['user'], env['hosts'][0], env['project']))

		# Create the media directory
		# Restore existing media if a subdirectory of the project, if it is outside the project make sure it is owned by nginx
		for site in sites:
			module = import_module(site['settings_module'])
			MEDIA_ROOT = fix_static_path(module.MEDIA_ROOT, site)
			sudo('mkdir -p %s' % MEDIA_ROOT)
			if MEDIA_ROOT.startswith('/srv/www/%s/' % env['project']):
				if exists('/srv/www/%smedia' % env['project']):
					# Delete the media directory but leave intermediate directories then restore with a move
					sudo('rm -rf %s' % MEDIA_ROOT)
					sudo('mv /srv/www/%smedia %s' % (env['project'], MEDIA_ROOT))
			else:
				sudo('chown -R %s:%s %s' % (NGINX_USER, NGINX_USER, MEDIA_ROOT))

			# Copy any additional webapp files
			if site.has_key('webapp') and site['webapp'].get('files'):
				from django.contrib.staticfiles import finders
				if hasattr(module, 'WEBAPP_ROOT'):
					WEBAPP_ROOT = fix_webapp_path(module.WEBAPP_ROOT)
					for filename in site['webapp']['files']:
						result = finders.find(filename)
						if not result:
							print red('Error: Could not find static file %s.' % filename)
							return
						result = result[0] if isinstance(result, (list, tuple)) else result
						put(result, os.path.join(WEBAPP_ROOT, filename), use_sudo=True, mode=0644)

def django_sync_dry_run(sites):
	"""Do an rsync dry run to see which files will be updated when deploying."""
	# e.g. To show just migrations: fab e:production x:dryrun | grep -v "^deleting" | grep -v "/$" | grep "^shared/migrations"
	excluded = _django_get_excluded(sites)
	excluded = ['--exclude="%s"' % ex for ex in excluded]
	local(SYNC_COMMAND % (' '.join(excluded), ' --dry-run -e "ssh -i %s"' % os.path.expanduser(env['key_filename']) if env.get('key_filename') else '', env['user'], env['hosts'][0], env['project']))

def django_sync_down(path=''):
	"""Syncs down stuff from the server. Wrap in the task decorator to enable. sync_down = task(sync_down)"""
	if path.startswith('/'):
		path = path[1:]
	if not path.endswith('/'):
		path += '/'
	local('rsync -avz --exclude=".*" --exclude="*.pyc" --exclude="*.sh" --exclude="*.db" %s@%s:/srv/www/%s%s ./%s' % (env['user'], env['hosts'][0], env['project'], path, path))

def deploy_webapp():
	"""Deploy a webapp to S3 with the prefix of WEBAPP_URL. If site is not specified, then the command will be run on all sites."""
	site = env.get('site')
	if not site:
		for site in env['sites'].values():
			env['site'] = site
			deploy_webapp()
		del env['site']
		return

	if site['type'] == SiteType.DJANGO:
		try:
			module = import_module(site['settings_module'])
			root = getattr(module, 'WEBAPP_ROOT', None)
			prefix = getattr(module, 'WEBAPP_URL', None)
			if not root:
				return
		except:
			print red('Error: Could not import webapp settings to deploy.')
			return
	else:
		if not site.has_key('webapp'):
			return
		root = site['webapp']['root']
		prefix = None

	# Build the webapp if needed
	task = site['webapp']['build_task'] if site.has_key('webapp') and site['webapp'].has_key('build_task') else None
	build_webapp(root, task)
	# Don't go any further if there is no bucket specified
	if not site.has_key('webapp') or not site['webapp'].has_key('bucket'):
		return
	# Clear the bucket
	if site['webapp'].get('clear'):
		delete_from_s3(site, site['webapp']['bucket'], prefix=prefix)
	# Upload the webapp files
	upload_to_s3(site, site['webapp']['bucket'], root, prefix=prefix)
	# Upload the static files found in other directories with the static file finder
	if site['webapp'].get('files'):
		if site['type'] == SiteType.DJANGO:
			from django.contrib.staticfiles import finders
			for filename in site['webapp']['files']:
				result = finders.find(filename)
				if not result:
					print red('Error: Could not find static file %s.' % filename)
					return
				result = result[0] if isinstance(result, (list, tuple)) else result
				upload_to_s3(site, site['webapp']['bucket'], result[:-len(filename)], (filename,), prefix=prefix)
		else:
			if type(site['webapp']['files']) is dict:
				for prefix in site['webapp']['files']:
					upload_to_s3(site, site['webapp']['bucket'], None, site['webapp']['files'][prefix], prefix=prefix)
			else:
				upload_to_s3(site, site['webapp']['bucket'], None, site['webapp']['files'], prefix=prefix)



import os

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.management import call_command

class Command(BaseCommand):
	help = 'Reset all migrations to just initial files. Also consider using the --update when doing schemamigrations.'

	def handle(self, *args, **options):
		if not hasattr(settings, 'DEBUG') or not settings.DEBUG:
			print 'This command is only valid for development, DEBUG is not True.'
			return
		for module in settings.SOUTH_MIGRATION_MODULES:
			os.system('rm -rf %s' % settings.SOUTH_MIGRATION_MODULES[module].replace('.', '/'))
			call_command('schemamigration', module, initial=True)
			if hasattr(settings, 'SOUTH_MIGRATION_DEPENDENCIES'):
				if settings.SOUTH_MIGRATION_DEPENDENCIES.has_key(module):
					dependsOnStr = ','.join(['("%s", "0001_initial")' % dependency for dependency in settings.SOUTH_MIGRATION_DEPENDENCIES[module]])
					os.system('sed -i "" -e "s/%s/%s/g" %s' % ('class Migration\\(SchemaMigration\\):', 'class Migration(SchemaMigration):\\n    depends_on = (%s,)\\n' % dependsOnStr, settings.SOUTH_MIGRATION_MODULES[module].replace('.', '/') + '/0001_initial.py'))



from optparse import make_option
import os
import subprocess
import sys

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.management import call_command

class Command(BaseCommand):
	help = 'Run schemamigration --auto for all apps under south.'

	def handle(self, *args, **options):
		for module in settings.SOUTH_MIGRATION_MODULES:
			print 'Migrating %s' % module
			result = subprocess.call([sys.argv[0], 'schemamigration', module, '--auto'])



import os
import tempfile
import urlparse

from fabric.api import put, sudo, hide, settings, env
from fabric.contrib.files import upload_template

from .service import Service
from ..shared import apt_get_install, get_template, fix_absolute_path, SiteType, chown, find_static
from ..shared import get_django_setting, get_static_root, get_static_url, get_media_root, get_media_url, get_webapp_root, get_webapp_url
from ..hooks import hook

_NGINX_SSL = """listen 443 ssl;\n\tssl_certificate %s;\n\tssl_certificate_key %s;"""
_NGINX_LOCATION = """location %s {\n\t\talias %s;\n\t\texpires 1d;\n\t}"""
_NGINX_LOCATION_DOMAIN = """location %s {\n\t\trewrite ^(.*)$ http://%s$1 permanent;\n\t}"""
_NGINX_WEBAPP_LOCATION = """location %s {\n\t\talias %s;\n\t\ttry_files $uri $uri/%s %s;\n\t}"""

def _get_domain(url):
	return urlparse.urlparse(url).netloc

def _get_path(url):
	return urlparse.urlparse(url).path

def _slash_append(path):
	if not path.endswith('/'):
		return path + '/'
	return path

def _slash_wrap(path):
	if path.startswith('/'):
		if path.endswith('/'):
			return path
		else:
			return path + '/'
	else:
		if path.endswith('/'):
			return '/' + path
		else:
			return '/%s/' % path

class Nginx(Service):
	name = 'nginx'
	script = 'nginx'

	def install(self):
		with hook('install %s' % self.name, self):
			apt_get_install('nginx-full')

	def config(self):
		with hook('config %s' % self.name, self):
			if self.settings:
				with tempfile.NamedTemporaryFile('w') as f:
					for section in self.settings:
						if isinstance(self.settings[section], dict):
							f.write('%s {\n' % section)
							for setting in self.settings[section]:
								f.write('\t%s %s;\n' % (setting, self.settings[section][setting]))
							f.write('}\n')
						elif type(self.settings[section]) is bool:
							f.write('%s %s;\n' % (section, 'on' if self.settings[section] else 'off'))
						else:
							f.write('%s %s;\n' % (section, str(self.settings[section])))
					f.flush()
					chown(put(f.name, '/etc/nginx/conf.d/fabric.conf', use_sudo=True, mode=0644))
		self.restart()

	def site_config(self, site):
		with hook('site config %s' % self.name, self, site):
			context = {
				'site': site['name'],
				'default_str': ' default_server' if self.settings.get('default') else '',
				'app_location': '/',
				'webapp_location': '',
				'error_page_str': '',
			}
			if self.settings.has_key('log_level'):
				context['log_level'] = ' ' + self.settings['log_level']
			else:
				context['log_level'] = ''
			if self.settings.get('ssl_cert') and self.settings.get('ssl_cert_key'):
				context['ssl_str'] = _NGINX_SSL % (self.settings['ssl_cert'], self.settings['ssl_cert_key'])
			else:
				context['ssl_str'] = ''
			if self.settings.has_key('location_settings') and isinstance(self.settings['location_settings'], (list, tuple)):
				context['location_settings_str'] = '\n\t\t'.join(['%s %s;' % setting for setting in self.settings['location_settings']])
			else:
				context['location_settings_str'] = ''

			if self.settings.has_key('server_settings') and isinstance(self.settings['server_settings'], (list, tuple)):
				context['server_settings_str'] = '\n\t\t'.join(['%s %s;' % setting for setting in self.settings['server_settings']])
			else:
				context['server_settings_str'] = ''

			if site['type'] == SiteType.DJANGO:
				# Django site setup
				context['location_settings_str'] = '\n\t\t'.join((context['location_settings_str'], 'uwsgi_pass unix:///var/run/uwsgi/app/%s/socket;' % site['name'], 'include uwsgi_params;'))
				context['allowed_hosts'] = ' '.join(get_django_setting(site, 'ALLOWED_HOSTS') or [site['name']])
				# Setup the static and media locations
				locations = []
				static_root = get_static_root(site)
				static_url = get_static_url(site)
				media_root = get_media_root(site)
				media_url = get_media_url(site)
				domain = _get_domain(static_url)
				if domain:
					locations.append(_NGINX_LOCATION_DOMAIN % (_slash_wrap(_get_path(static_url)), domain))
				else:
					locations.append(_NGINX_LOCATION % (_slash_wrap(static_url), _slash_append(static_root)))
				domain = _get_domain(media_url)
				if domain:
					locations.append(_NGINX_LOCATION_DOMAIN % (_slash_wrap(_get_path(media_url)), domain))
				else:
					locations.append(_NGINX_LOCATION % (_slash_wrap(media_url), _slash_append(media_root)))
				# Add any custom locations
				if self.settings.has_key('custom_locations'):
					if isinstance(self.settings['custom_locations'], (tuple, list)):
						for location in self.settings['custom_locations']:
							locations.append(location)
					else:
						locations.append(self.settings['custom_locations'])
				context['static_locations'] = '\n\n\t'.join(locations)
				# Configure the webapp if necessary
				webapp_root = get_webapp_root(site)
				if webapp_root:
					webapp_url = _slash_wrap(get_webapp_url(site))
					webapp_index = get_django_setting(site, 'WEBAPP_INDEX') or 'index.html'
					if webapp_url == '/':
						context['app_location'] = '@%s-app' % site['name'].replace('.', '-')
						context['webapp_location'] = _NGINX_WEBAPP_LOCATION % (webapp_url, _slash_append(webapp_root), webapp_index, context['app_location'])
					else:
						context['app_location'] = '/'
						context['webapp_location'] = _NGINX_WEBAPP_LOCATION % (webapp_url, _slash_append(webapp_root), webapp_index, '=404')
					# Configure the error pages if present
					error_pages = []
					if not env.get('vagrant'):
						for status_code in (400, 401, 403, 404, 500):
							static_name = '%d.html' % status_code
							result = find_static(site, static_name)
							if result:
								error_pages.append('error_page %d %s;' % (status_code, os.path.join(static_url, static_name)))
					context['error_page_str'] = '\n\t'.join(error_pages)
			else:
				# Not a django site
				context['allowed_hosts'] = site['name']
				context['static_locations'] = ''

			if self.settings.get('ssl_only'):
				nginx_template = get_template('nginx-site-https.conf')
			else:
				nginx_template = get_template('nginx-site.conf')
			upload_template(nginx_template, '/etc/nginx/sites-available/%s.conf' % site['name'], context=context, use_sudo=True, mode=0644)
			sudo('chown root:root /etc/nginx/sites-available/%s.conf' % site['name'])
			# If site type is NGINX enable it right away because there is no deployment process for it
			if site['type'] == SiteType.NGINX:
				with hide('warnings'), settings(warn_only=True):
					sudo('ln -sf /etc/nginx/sites-available/%s.conf /etc/nginx/sites-enabled/%s.conf' % (site['name'], site['name']))
			# If the site is the default, then remove the default that comes with nginx
			if self.settings.get('default'):
				sudo('rm -f /etc/nginx/sites-enabled/default')
		self.restart()



from importlib import import_module
import tempfile

from fabric.api import run, sudo, cd, put, env, settings
from fabric.context_managers import shell_env
from fabric.contrib.files import append

from .service import Service
from ..hooks import hook
from ..shared import apt_get_install, pip_install, find_service

_POSTGRES_USER = 'postgres'

class Postgres(Service):
	name = 'postgres'
	script = 'postgresql'

	def __init__(self, settings=None, postgis=False):
		self.settings = settings
		self.postgis = postgis

	def install(self):
		with hook('install %s' % self.name, self):
			apt_get_install('postgresql')
			if self.postgis:
				self.install_postgis()

	def install_postgis(self):
		apt_get_install('postgresql-server-dev-all', 'libpq-dev', 'libxml2', 'libxml2-dev')

		# Install geos
		run('wget --no-clobber http://archive.ubuntu.com/ubuntu/pool/universe/g/geos/geos_3.3.3.orig.tar.gz')
		run('tar -xzf geos_3.3.3.orig.tar.gz')
		with cd('geos-3.3.3'):
			run('./configure')
			run('make')
			sudo('make install')

		# Install proj.4
		# https://docs.djangoproject.com/en/dev/ref/contrib/gis/install/
		run('wget --no-clobber http://download.osgeo.org/proj/proj-4.8.0.tar.gz')
		run('wget --no-clobber http://download.osgeo.org/proj/proj-datumgrid-1.5.tar.gz')
		run('tar -xzf proj-4.8.0.tar.gz')
		with cd('proj-4.8.0/nad'):
			run('tar -xzf ../../proj-datumgrid-1.5.tar.gz')
			with cd('..'):
				run('./configure')
				run('make')
				sudo('make install')

		# Install gdal
		run('wget --no-clobber http://download.osgeo.org/gdal/gdal-1.9.2.tar.gz')
		run('tar -xzf gdal-1.9.2.tar.gz')
		with cd('gdal-1.9.2'):
			run('./configure')
			run('make')
			sudo('make install')
			sudo('ldconfig')

		run('wget --no-clobber http://download.osgeo.org/postgis/source/postgis-2.0.3.tar.gz')
		run('tar -xzf postgis-2.0.3.tar.gz')
		with cd('postgis-2.0.3'):
			run('./configure')
			run('make')
			sudo('make install')

	def config(self):
		with hook('config %s' % self.name, self):
			if self.settings:
				with tempfile.NamedTemporaryFile('w') as f:
					for setting in self.settings:
						f.write('%s = %s\n' % (setting, self.settings[setting]))
					f.flush()
					put(f.name, '/etc/postgresql/9.1/main/fabric.conf', use_sudo=True, mode=0644)
					sudo('chown %s:%s /etc/postgresql/9.1/main/fabric.conf' % (_POSTGRES_USER, _POSTGRES_USER))
					append('/etc/postgresql/9.1/main/postgresql.conf', "\n\ninclude 'fabric.conf'\n", use_sudo=True)
		self.restart()

	def site_install(self, site):
		with hook('site install %s' % self.name, self, site):
			if self.postgis:
				# Install PostGIS also on the site if separate from the server
				if find_service(self.name) is None:
					self.install_postgis()
			# Install python postgresql support
			apt_get_install('python-dev', 'postgresql-server-dev-all', 'postgresql-client')
			pip_install('psycopg2')

	def site_config(self, site):
		answer = raw_input('Running this postgres site configuration will drop any existing database. Are you sure? (Y/N): ')
		if answer.lower() != 'y':
			return
		answer = raw_input('Are you absolutely sure? (Y/N): ')
		if answer.lower() != 'y':
			return
		with hook('site config %s' % self.name, self, site):
			# Create the user for django to access the database with
			module = import_module(site['settings_module'])
			DATABASES = module.DATABASES
			remote_db = True
			if find_service(self.name) is None:
				# For a remote database
				connect_args = '--username=%s --host=%s' % (DATABASES['default']['USER'], DATABASES['default']['HOST'])
				if DATABASES['default'].get('PORT'):
					connect_args += ' --port=%s' % str(DATABASES['default']['PORT'])
				user = None
			else:
				# For a local database setup the users
				connect_args = ''
				user = _POSTGRES_USER
				with settings(warn_only=True):
					sudo('createuser --createdb --no-superuser --no-createrole %s' % DATABASES['default']['USER'], user=user)
					sudo("psql -c \"ALTER USER %s WITH PASSWORD '%s';\"" % (DATABASES['default']['USER'], DATABASES['default']['PASSWORD']), user=user)
					sudo('createuser --createdb --no-superuser --no-createrole %s' % env.user, user=user)
			with shell_env(PGPASSWORD=DATABASES['default']['PASSWORD']):
				with settings(warn_only=True):
					sudo('dropdb %s %s' % (connect_args, DATABASES['default']['NAME']), user=user)
				sudo('createdb %s %s' % (connect_args, DATABASES['default']['NAME']), user=user)
				if self.postgis:
					sudo("psql %s %s -c 'CREATE EXTENSION postgis;'" % (connect_args, DATABASES['default']['NAME']), user=user)
					sudo("psql %s %s -c 'CREATE EXTENSION postgis_topology;'" % (connect_args, DATABASES['default']['NAME']), user=user)
					# http://tmbu.blogspot.com/2012/11/postgres-postgis-django-os-x-and.html
					# http://stackoverflow.com/questions/4737982/django-avoids-creating-pointfield-in-the-database-when-i-run-python-manage-py-sy
					sudo("psql %s %s -c 'GRANT ALL PRIVILEGES ON geometry_columns TO %s;'" % (connect_args, DATABASES['default']['NAME'], DATABASES['default']['USER']), user=user)
					sudo("psql %s %s -c 'GRANT ALL PRIVILEGES ON spatial_ref_sys TO %s;'" % (connect_args, DATABASES['default']['NAME'], DATABASES['default']['USER']), user=user)



from fabric.api import sudo, run
from fabric.contrib.files import exists

from .service import Service
from ..hooks import hook
from ..shared import apt_get_install, pip_install

class GeoIP(Service):
	"""Database to convert ip addresses into locations."""
	name = 'geoip'
	script = None

	def install(self):
		with hook('install %s' % self.name, self):
			if not exists('/usr/local/geoip'):
				apt_get_install('libgeoip1')
				run('wget --no-clobber http://www.maxmind.com/download/geoip/database/GeoLiteCity.dat.gz')
				run('gunzip GeoLiteCity.dat.gz')
				sudo('mkdir /usr/local/geoip')
				sudo('mv GeoLiteCity.dat /usr/local/geoip/')

	def site_install(self, site):
		with hook('site install %s' % self.name, self, site):
			pip_install()



import urlparse

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpResponsePermanentRedirect
from django.http.request import validate_host

def require_superuser(func):
	"""Limit view to superusers only."""
	def require_superuser_fun(request, *args, **kwargs):
		if not request.user.is_superuser or not request.user.is_active:
			return redirect_to_login(request.get_full_path())
		return func(request, *args, **kwargs)
	return require_superuser_fun

def require_staff(func):
	"""Limit view to staff only.
	
	The django.contrib.admin.views.decorators.staff_member_required decorator will only redirect to the admin login page.
	"""
	def require_staff_fun(request, *args, **kwargs):
		if not request.user.is_staff or not request.user.is_active:
			return redirect_to_login(request.get_full_path())
		return func(request, *args, **kwargs)
	return require_staff_fun

def require_superuser_403(func):
	"""Limit view to superusers only or raise a 403."""
	def require_superuser_fun(request, *args, **kwargs):
		if not request.user.is_superuser or not request.user.is_active:
			raise PermissionDenied()
		return func(request, *args, **kwargs)
	return require_superuser_fun

def require_staff_403(func):
	"""Limit view to staff only or raise a 403."""
	def require_staff_fun(request, *args, **kwargs):
		if not request.user.is_staff or not request.user.is_active:
			raise PermissionDenied()
		return func(request, *args, **kwargs)
	return require_staff_fun

def require_referer(*netlocs):
	"""Limit a view to a specific list of referers. If unspecified, settings.ALLOWED_HOSTS is used."""
	def require_referer_decorator(func):
		def require_referer_fun(request, *args, **kwargs):
			if settings.DEBUG:
				return func(request, *args, **kwargs)
			referer = request.META.get('HTTP_REFERER', '')
			if not netlocs:
				if len(settings.ALLOWED_HOSTS):
					if validate_host(urlparse.urlparse(referer).netloc, settings.ALLOWED_HOSTS):
						return func(request, *args, **kwargs)
			else:
				if urlparse.urlparse(referer).netloc in netlocs:
					return func(request, *args, **kwargs)
			raise PermissionDenied()
		return require_referer_fun
	return require_referer_decorator

def require_https(func):
	"""Redirect to https if the view is being requested with http."""
	def require_https_fun(request, *args, **kwargs):
		if settings.DEBUG:
			return func(request, *args, **kwargs)
		if not request.is_secure():
			request_url = request.build_absolute_uri(request.get_full_path())
			secure_url = request_url.replace('http://', 'https://')
			return HttpResponsePermanentRedirect(secure_url)
		return func(request, *args, **kwargs)
	return require_https_fun

# when using with toastr.js, all types are supported besides debug
# this can be overriden https://docs.djangoproject.com/en/dev/ref/contrib/messages/
def add_message(level, message):
	"""Add a message to a view."""
	def add_message_decorator(func):
		def add_message_fun(request, *args, **kwargs):
			messages.add_message(request, level, message)
			return func(request, *args, **kwargs)
		return add_message_fun
	return add_message_decorator

def require_basic_auth(username, password, realm):
	"""Requires the browser to do basic authentication with the user/password pair specified."""
	def require_basic_auth_decorator(func):
		def require_basic_auth_fun(request, *args, **kwargs):
			if request.META.has_key('HTTP_AUTHORIZATION'):
				auth = [i for i in request.META['HTTP_AUTHORIZATION'].split(' ') if i]
				if auth[0].lower() == 'basic' and auth[1].decode('base64') == '%s:%s' % (username, password):
					return func(request, *args, **kwargs)
			response = HttpResponse(status=401)
			response['WWW-Authenticate'] = 'Basic realm="%s"' % realm
			return response
		return require_basic_auth_fun
	return require_basic_auth_decorator



import tempfile

from fabric.api import put, sudo
from fabric.contrib.files import append

from .service import Service
from ..hooks import hook
from ..shared import apt_get_install, pip_install, chown

class Redis(Service):
	name = 'redis'
	script = 'redis-server'

	def install(self):
		with hook('install %s' % self.name, self):
			apt_get_install('redis-server')

	def config(self):
		with hook('config %s' % self.name, self):
			if self.settings:
				with tempfile.NamedTemporaryFile('w') as f:
					for setting in self.settings:
						if self.settings[setting]:
							f.write('%s %s\n' % (setting, self.settings[setting]))
						else:
							f.write('%s\n' % setting)
					f.flush()
					chown(put(f.name, '/etc/redis/fabric.conf', use_sudo=True, mode=0644))
					append('/etc/redis/redis.conf', 'include /etc/redis/fabric.conf', use_sudo=True)
		self.restart()

	def site_install(self, site):
		with hook('site install %s' % self.name, self, site):
			pip_install(site, 'redis', 'django-redis')



from django.conf.urls import url
from django.views.static import serve

def static_serve_url(static_url, static_root, name=None, view=serve):
	static_url = static_url.strip('/')
	if static_url:
		static_url = '^%s/(?P<path>.*)$' % static_url
	else:
		static_url = '^(?P<path>.*)$'
	return url(static_url, view, {'document_root': static_root}, name)



import StringIO

def format_ini(values, quotes=True):
	output = StringIO.StringIO()
	for section in values:
		print >>output, '[%s]' % section
		for key, value in values[section].iteritems():
			value = str(value)
			if quotes and (';' in value or '=' in value or '#' in value or ' ' in value):
				value = '"%s"' % value
			print >>output, '%s=%s' % (key, value)
		print ''
	value = output.getvalue()
	output.close()
	return value



from fabric.api import settings, sudo
from fabric.contrib.files import exists

from .decorators import before, after
from ..shared import apt_get_install

@before('pip install pil')
def install_img_libs():
	apt_get_install('libjpeg-dev', 'libpng-dev', 'libfreetype6-dev')

# Setup image libraries properly
# http://stackoverflow.com/questions/7648200/pip-install-pil-e-tickets-1-no-jpeg-png-support

@after('apt-get install libjpeg-dev')
def setup_libjpeg():
	with settings(warn_only=True):
		if exists('/usr/lib/x86_64-linux-gnu/libjpeg.so'):
			sudo('ln -s /usr/lib/x86_64-linux-gnu/libjpeg.so /usr/lib/libjpeg.so')
		if exists('/usr/lib/x86_64-linux-gnu/libz.so'):
			sudo('ln -s /usr/lib/x86_64-linux-gnu/libz.so /usr/lib/libz.so')

@after('apt-get install libfreetype6-dev')
def setup_libfreetype6():
	with settings(warn_only=True):
		if exists('/usr/lib/x86_64-linux-gnu/libfreetype.so'):
			sudo('ln -s /usr/lib/x86_64-linux-gnu/libfreetype.so /usr/lib/libfreetype.so')
		if exists('/usr/lib/x86_64-linux-gnu/libz.so'):
			sudo('ln -s /usr/lib/x86_64-linux-gnu/libz.so /usr/lib/libz.so')



from optparse import make_option
import os

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError

class Command(BaseCommand):
	help = 'Reset the database to just initial data, also calls resetmigrations. Only to be used on development sqlite databases.'
	option_list = BaseCommand.option_list + (
			make_option('--preserve', '-p',
				action='store_true',
				dest='preserve',
				default=False,
				help='Preserve the database data through resyncing, remigrating.'),
		)

	def handle(self, *args, **options):
		if not hasattr(settings, 'DEBUG') or not settings.DEBUG:
			print 'This command is only valid for development, DEBUG is not True.'
			return
		if settings.DATABASES['default']['ENGINE'].find('sqlite') == -1:
			print 'This command is only valid for development with sqlite databases.'
			return
		import pdb; pdb.set_trace()
		file = settings.DATABASES['default']['NAME']
		dataFile = file + '.json'
		if options['preserve']:
			os.system('python manage.py dumpdata --exclude south --exclude contenttypes --exclude auth.Permission --exclude admin.LogEntry > %s' % dataFile)
		call_command('resetmigrations')
		os.system('rm %s' % file)
		if options['preserve']:
			os.system('python manage.py syncdb --noinput --no-initial-data')
			os.system('python manage.py migrate --noinput --no-initial-data')
			call_command('loaddata', dataFile)
			os.system('rm %s' % dataFile)
		else:
			os.system('python manage.py syncdb --noinput')
			os.system('python manage.py migrate --noinput')



from random import randint

from django.template.defaultfilters import slugify

# instance must have a field named slug for this method to work
def unique_slug_generator(instance, text, suffix_length=9, max_length=50):
	text = slugify(text)[:max_length - suffix_length]
	num = str(randint(10 ** (suffix_length - 1), (10 ** suffix_length) - 1))
	potential_slugs = [text + '-'[:x] + num[:x] for x in range(suffix_length + 1)]
	taken_slugs = list(instance.__class__.objects.filter(slug__in=potential_slugs).values_list('slug', flat=True))
	for slug in potential_slugs:
		try:
			taken_slugs.index(slug)
		except:
			yield slug



from datetime import tzinfo

from django.contrib.auth import get_user_model
from django.http import HttpRequest
from django.shortcuts import redirect
from django.utils import timezone

class AssumeMiddleware(object):
	"""If enabled, each request object will have it's user attribute set to the assumed user.

	The following additional attributes will also be added to the request.user object:
	user.from_user - the actual logged in staff user
	user.is_assumed - True when the user is assumed
	"""

	def process_request(self, request):
		assume = request.GET.get('assume', None)
		unassume = request.GET.get('unassume', None)
		
		if assume:
			request.session['assume_username'] = assume
			if request.META.has_key('HTTP_REFERER') and request.META['HTTP_REFERER']:
				request.session['assume_referer'] = request.META['HTTP_REFERER']
		
		if not unassume and request.session.has_key('assume_username') and request.user.is_staff:
			try:
				from_user = request.user
				request.user = get_user_model().objects.get(username=request.session['assume_username'])
				request.user.is_assumed = True
				request.user.from_user = from_user
			except:
				unassume = True

		if unassume:
			if request.session.has_key('assume_username'):
				del request.session['assume_username']
			if request.session.has_key('assume_referer'):
				request.assume_referer = request.session['assume_referer']
				del request.session['assume_referer']

	def process_view(self, request, view_func, view_args, view_kwargs):
		if request.GET.has_key('assume'):
			q = request.GET.copy()
			del q['assume']
			request.GET = q
			request.META['QUERY_STRING'] = q.urlencode()
			return redirect(request.build_absolute_uri())
		if request.GET.has_key('unassume'):
			if hasattr(request, 'assume_referer') and request.assume_referer:
				# Set in the process_request method
				return redirect(request.assume_referer)
			else:
				q = request.GET.copy()
				del q['unassume']
				request.GET = q
				request.META['QUERY_STRING'] = q.urlencode()
				return redirect(request.build_absolute_uri())

class VarOffset(tzinfo):

	def __init__(self):
		self.__offset = 0

	def utcoffset(self, dt):
		return self.__offset

	def tzname(self, dt):
		return ''

	def dst(self, dt):
		return timezone.ZERO

	@property
	def offset(self):
		return self.__offset

	@offset.setter
	def offset(self, value):
		self.__offset = value

class TimezoneMiddleware(object):
	"""Automatically sets based on the request.user's timezone. MUST be installed after the AuthenticationMiddleware."""

	def __init__(self):
		self.__timezone = VarOffset()

	def process_request(self, request):
		offset = getattr(request.user,'timezone', None)
		if offset is not None:
			self.__timezone.offset = offset
			timezone.activate(self.__timezone)

	def process_response(self, request, response):
		timezone.deactivate()



from django import template
from django.template.defaultfilters import stringfilter
from django.template.defaultfilters import urlize
from django.utils.safestring import mark_safe

register = template.Library()

@register.filter(is_safe=True, needs_autoescape=True)
@stringfilter
def urlizeblank(value, autoescape=None):
	"""Urlize the string and set the url targets to _blank."""
	return mark_safe(urlize(value, autoescape).replace('<a', '<a target="_blank"'))



from django import VERSION
from django.db import models

from .encryption import encrypt, decrypt

# Note: in 1.8 SubfieldBase is deprecated and from_db_value was added
if VERSION[0] > 1 or VERSION[1] > 7:
	_metaclass = type
else:
	_metaclass = models.SubfieldBase

class EncryptedField(models.CharField):
	__metaclass__ = _metaclass
	description = "An encrypted CharField using AES encryption."

	def __init__(self, *args, **kwargs):
		kwargs['serialize'] = False
		kwargs['editable'] = False
		kwargs['blank'] = True
		kwargs.setdefault('max_length', 255)
		super(EncryptedField, self).__init__(*args, **kwargs)

	def from_db_value(self, value, expression, connection, context):
		if value and value.startswith('AES$'):
			return decrypt(value)
		return value

	def to_python(self, value):
		if value and value.startswith('AES$'):
			return super(EncryptedField, self).to_python(decrypt(value))
		return super(EncryptedField, self).to_python(value)

	def get_prep_value(self, value):
		if value:
			return encrypt(value)
		return value

class EncryptedTextField(models.TextField):
	__metaclass__ = _metaclass
	description = "An encrypted TextField using AES encryption."

	def __init__(self, *args, **kwargs):
		kwargs['serialize'] = False
		kwargs['editable'] = False
		kwargs['blank'] = True
		super(EncryptedTextField, self).__init__(*args, **kwargs)

	def from_db_value(self, value, expression, connection, context):
		if value and value.startswith('AES$'):
			return decrypt(value)
		return value

	def to_python(self, value):
		if value and value.startswith('AES$'):
			return super(EncryptedTextField, self).to_python(decrypt(value))
		return super(EncryptedTextField, self).to_python(value)

	def get_prep_value(self, value):
		if value:
			return encrypt(value)
		return value



import base64
import random

from django.conf import settings

def _pad(plaintext, block_size=16):
	padding = block_size - (len(plaintext) % block_size)
	return plaintext + '\0' + ''.join([chr(random.randint(ord('!'), ord('~'))) for x in range(padding - 1)])

def encrypt(plaintext, key=None, cipher='aes-256-cbc'):
	if not key:
		key = settings.SECRET_KEY
	try:
		from Crypto.Cipher import AES
		cipher = AES.new(key[:32])
		return 'AES$' + base64.b64encode(cipher.encrypt(_pad(plaintext)))
	except:
		import commands
		return commands.getoutput('printf "%s" | openssl %s -e -base64 -A -salt -k "%s"' % (plaintext, cipher, key))

def decrypt(ciphertext, key=None, cipher='aes-256-cbc'):
	if not key:
		key = settings.SECRET_KEY
	try:
		from Crypto.Cipher import AES
		cipher = AES.new(key[:32])
		return cipher.decrypt(base64.b64decode(ciphertext[4:])).split('\0')[0]
	except:
		import commands
		return commands.getoutput('printf "%s" | openssl %s -d -base64 -A -k "%s"' % (ciphertext, cipher, key))



from fabric.api import sudo

from ..hooks import hook

class Service(object):

	def __init__(self, *args, **kwargs):
		self.settings = {}
		for settings_dict in args:
			self.settings.update(settings_dict)
		self.settings.update(kwargs)

	def copy(self, **kwargs):
		c = self.__class__(**self.settings)
		c.settings.update(kwargs)
		return c

	def __copy__(self):
		return self.copy()

	def __deepcopy__(self, memo):
		return self.copy()

	# All the setup commands

	def install(self):
		pass

	def config(self):
		pass

	def site_install(self, site):
		pass

	def site_config(self, site):
		pass

	# Functions to alter the service running state

	def start(self):
		if hasattr(self, 'script'):
			with hook('start %s' % self.name, self):
				sudo('service %s start' % self.script, pty=False)

	def stop(self):
		if hasattr(self, 'script'):
			with hook('stop %s' % self.name, self):
				sudo('service %s stop' % self.script, pty=False)

	def restart(self):
		if hasattr(self, 'script'):
			with hook('restart %s' % self.name, self):
				sudo('service %s restart' % self.script, pty=False)

	# Additional settings to use when deploying the site

	def get_site_settings(self, site):
		return {}



__bold = '\033[1m'
__reset = '\033[0m'

class Color:
	BLACK, RED, GREEN, YELLOW, BLUE, PURPLE, TEAL, WHITE = range(30,38)

def __color_str(msg, color, bold, background=None):
	if background:
		background += 10
		if bold:
			return '\033[1;%d;%dm%s%s' % (color, background, msg, __reset)
		else:
			return '\033[%d;%dm%s%s' % (color, background, msg, __reset)
	else:
		if bold:
			return '\033[1;%dm%s%s' % (color, msg, __reset)
		else:
			return '\033[%dm%s%s' % (color, msg, __reset)

def bold(msg):
	return '%s%s%s' % (__bold, msg, __reset)

def black(msg, bold=False, background=None):
	return __color_str(msg, Color.BLACK, bold, background)

def red(msg, bold=False, background=None):
	return __color_str(msg, Color.RED, bold, background)

def green(msg, bold=False, background=None):
	return __color_str(msg, Color.GREEN, bold, background)

def yellow(msg, bold=False, background=None):
	return __color_str(msg, Color.YELLOW, bold, background)

def blue(msg, bold=False, background=None):
	return __color_str(msg, Color.BLUE, bold, background)

def purple(msg, bold=False, background=None):
	return __color_str(msg, Color.PURPLE, bold, background)

def teal(msg, bold=False, background=None):
	return __color_str(msg, Color.TEAL, bold, background)

def white(msg, bold=False, background=None):
	return __color_str(msg, Color.WHITE, bold, background)



from .memcached import Memcached
from .mysql import MySQL
from .nginx import Nginx
from .postgres import Postgres
from .redis import Redis
from .uwsgi import UWSGI
from .s3 import S3
from .geoip import GeoIP
from .cron import Cron

__all__ = ['Memcached', 'MySQL', 'Nginx', 'Postgres', 'Redis', 'UWSGI', 'S3', 'GeoIP', 'Cron']



from fabric.api import run, sudo, cd

from ..shared import apt_get_install, get_django_setting

_GEOS_URL = 'http://archive.ubuntu.com/ubuntu/pool/universe/g/geos/geos_3.3.3.orig.tar.gz'
_GEOS_DIR = 'geos-3.3.3'
_PROJ4_URL = 'http://download.osgeo.org/proj/proj-4.8.0.tar.gz'
_PROJ4_DIR = 'proj-4.8.0/nad'
_PROJ4_DATUM_URL = 'http://download.osgeo.org/proj/proj-datumgrid-1.5.tar.gz'
_GDAL_URL = 'http://download.osgeo.org/gdal/gdal-1.9.2.tar.gz'
_GDAL_DIR = 'gdal-1.9.2'
_POSTGIS_URL = 'http://download.osgeo.org/postgis/source/postgis-2.0.3.tar.gz'
_POSTGIS_DIR = 'postgis-2.0.3'

def install_postgis():
	apt_get_install('postgresql-server-dev-all', 'libpq-dev', 'libxml2', 'libxml2-dev')

	# Install geos
	run('wget --no-clobber %s' % _GEOS_URL)
	run('tar -xzf %s' % _GEOS_URL.split('/')[-1])
	with cd(_GEOS_DIR):
		run('./configure')
		run('make')
		sudo('make install')

	# Install proj.4
	# https://docs.djangoproject.com/en/dev/ref/contrib/gis/install/
	run('wget --no-clobber %s' % _PROJ4_URL)
	run('wget --no-clobber %s' % _PROJ4_DATUM_URL)
	run('tar -xzf %s' % _PROJ4_URL.split('/')[-1])
	with cd(_PROJ4_DIR):
		run('tar -xzf ../../%s' % _PROJ4_DATUM_URL.split('/')[-1])
		with cd('..'):
			run('./configure')
			run('make')
			sudo('make install')

	# Install gdal
	run('wget --no-clobber %s' % _GDAL_URL)
	run('tar -xzf %s' % _GDAL_URL.split('/')[-1])
	with cd(_GDAL_DIR):
		run('./configure')
		run('make')
		sudo('make install')
		sudo('ldconfig')

	run('wget --no-clobber %s' % _POSTGIS_URL)
	run('tar -xzf %s' % _POSTGIS_URL.split('/')[-1])
	with cd(_POSTGIS_DIR):
		run('./configure')
		run('make')
		sudo('make install')

def site_config_postgis(postgres, site):
	database = get_django_setting(site, 'DATABASES')['default']
	postgres.execute_sql('CREATE EXTENSION postgis;', site)
	postgres.execute_sql('CREATE EXTENSION postgis_topology;', site)
	# http://tmbu.blogspot.com/2012/11/postgres-postgis-django-os-x-and.html
	# http://stackoverflow.com/questions/4737982/django-avoids-creating-pointfield-in-the-database-when-i-run-python-manage-py-sy
	postgres.execute_sql('GRANT ALL PRIVILEGES ON geometry_columns TO %s;' % database['USER'], site)
	postgres.execute_sql('GRANT ALL PRIVILEGES ON spatial_ref_sys TO %s;' % database['USER'], site)



from django import template

register = template.Library()

@register.filter
def mul(value, arg):
	"""Multiplies the arg and the value."""
	if type(value) is str or type(value) is unicode:
		value = int(value)
	return value * arg

@register.filter
def sub(value, arg):
	"""Subtracts the arg from the value."""
	if type(value) is str or type(value) is unicode:
		value = int(value)
	return value - arg

@register.filter
def div(value, arg):
	"""Divides the value by the arg."""
	if type(value) is str or type(value) is unicode:
		value = int(value)
	if arg:
		return value / arg
	else:
		return 0

@register.filter
def mod(value, arg):
	"""Modulo the value by the arg."""
	if type(value) is str or type(value) is unicode:
		value = int(value)
	return value % arg



import tempfile

from fabric.api import put, sudo
from fabric.contrib.files import append, sed

from .service import Service
from ..formats import format_ini
from ..hooks import hook
from ..shared import WWW_USER, pip_install, get_template, get_project_directory, get_python_interpreter, chown

_CONFIG_FILE = '/etc/supervisor/supervisor.conf'

django_management_program = {
	'numprocs': '4',
	'process_name': '%(program_name)s_%(process_num)02d',
	'autostart': 'true',
	'autorestart': 'true',
	'user': WWW_USER
}

# NOTE: supervisor doesn't support quotes around the ini values in config files

class Supervisor(Service):
	name = 'supervisor'
	script = 'supervisor'

	def install(self):
		with hook('install %s' % self.name, self):
			pip_install(None, self.name)
			# To run automatically at startup with ubuntu and other systems:
			# http://serverfault.com/questions/96499/how-to-automatically-start-supervisord-on-linux-ubuntu
			chown(put(get_template('supervisor-upstart.conf'), '/etc/init/supervisor.conf', use_sudo=True, mode=0644))
			# Start the default configuration, with logging and pid file location changed to be more like nginx and other services
			sudo('mkdir -p /etc/supervisor')
			sudo('mkdir -p /var/log/supervisor')
			sudo('echo_supervisord_conf > ' + _CONFIG_FILE)
			sed(_CONFIG_FILE, '/tmp/supervisord.log', '/var/log/supervisor/supervisor.log', use_sudo=True)
			sed(_CONFIG_FILE, '/tmp/supervisord.pid', '/var/run/supervisor/supervisor.pid', use_sudo=True)
			sed(_CONFIG_FILE, '/tmp/supervisor.sock', '/var/run/supervisor/supervisor.sock', use_sudo=True)

	def config(self):
		with hook('config %s' % self.name, self):
			if self.settings:
				import StringIO
				chown(put(StringIO.StringIO(format_ini(self.settings, quotes=False)), '/etc/supervisor/supervisor-fabric.conf', use_sudo=True, mode=0644))
				append(_CONFIG_FILE, '[include]', use_sudo=True)
				append(_CONFIG_FILE, 'files=/etc/supervisor/supervisor-fabric.conf', use_sudo=True)
		self.restart()

	def site_config(self, site):
		# Any supervisor configuration can be used but shortcuts are provided for Django
		# If settings has 'program:name': 'management command' this will be expanded with default settings and correctly resolved to the project/virtualenv
		# Likewise if a 'program:name': { 'command': 'manage.py management command', ... } is used this will also be resolved to the project/virtualenv
		with hook('site config %s' % self.name, self, site):
			if self.settings:
				# Process shortcuts on setting up rq management commands
				for program, config in self.settings.items():
					if program.startswith('program:'):
						if isinstance(config, (str, unicode)):
							command = config
							config = self.settings[program] = django_management_program.copy()
							config['command'] = 'manage.py ' + command
						# Expand a manage.py command to use the correct python interpreter
						command = config.get('command', '')
						if command.startswith('manage.py'):
							config['command'] = get_python_interpreter() + ' ' + command
							config['directory'] = get_project_directory()
				import StringIO
				file_name = '/etc/supervisor/%s.conf' % site['name']
				chown(put(StringIO.StringIO(format_ini(self.settings, quotes=False)), file_name, use_sudo=True, mode=0644))
				append(_CONFIG_FILE, '[include]', use_sudo=True)
				append(_CONFIG_FILE, 'files=%s' % file_name, use_sudo=True)
		self.restart()



from fabric.api import sudo

from ..hooks import hook

class Service(object):

	def __init__(self, settings=None):
		self.settings = {} if settings is None else settings

	def install(self):
		pass

	def config(self):
		pass

	def site_install(self, site):
		pass

	def site_config(self, site):
		pass

	def start(self):
		if hasattr(self, 'script'):
			with hook('start %s' % self.name, self):
				sudo('service %s start' % self.script, pty=False)

	def stop(self):
		if hasattr(self, 'script'):
			with hook('stop %s' % self.name, self):
				sudo('service %s stop' % self.script, pty=False)

	def restart(self):
		if hasattr(self, 'script'):
			with hook('restart %s' % self.name, self):
				sudo('service %s restart' % self.script, pty=False)
